#!/usr/bin/env python3
"""
Price Alarm Manager

Manages price alarms for tokens with persistence and monitoring integration.
"""

import json
import os
import logging
from datetime import datetime
from typing import Dict, List, Any, Optional

logger = logging.getLogger(__name__)

class AlarmManager:
    """Manages price alarms with persistence and monitoring."""
    
    def __init__(self, alarms_file: str = "price_alarms.json"):
        """Initialize the alarm manager."""
        self.alarms_file = alarms_file
        self.data = self._load_alarms()
        
        # Initialize if first run
        if not self.data:
            self._initialize_alarms()
    
    def _load_alarms(self) -> Dict[str, Any]:
        """Load alarms from file."""
        try:
            if os.path.exists(self.alarms_file):
                with open(self.alarms_file, 'r') as f:
                    return json.load(f)
        except Exception as e:
            logger.error(f"Error loading alarms: {e}")
        return {}
    
    def _save_alarms(self):
        """Save alarms to file."""
        try:
            with open(self.alarms_file, 'w') as f:
                json.dump(self.data, f, indent=2, default=str)
        except Exception as e:
            logger.error(f"Error saving alarms: {e}")
    
    def _initialize_alarms(self):
        """Initialize alarms structure."""
        self.data = {
            'next_id': 1,
            'alarms': [],
            'triggered_alarms': [],
            'last_update': datetime.now().isoformat()
        }
        self._save_alarms()
    
    def create_alarm(self, token: str, target_price: float, current_price: float) -> Dict[str, Any]:
        """Create a new price alarm."""
        # Determine direction based on current vs target price
        direction = "above" if target_price > current_price else "below"
        
        alarm = {
            'id': self.data['next_id'],
            'token': token.upper(),
            'target_price': target_price,
            'current_price_when_set': current_price,
            'direction': direction,
            'status': 'active',
            'created_at': datetime.now().isoformat(),
            'triggered_at': None
        }
        
        self.data['alarms'].append(alarm)
        self.data['next_id'] += 1
        self.data['last_update'] = datetime.now().isoformat()
        self._save_alarms()
        
        logger.info(f"Created alarm {alarm['id']}: {token} @ ${target_price} ({direction})")
        return alarm
    
    def get_alarm_by_id(self, alarm_id: int) -> Optional[Dict[str, Any]]:
        """Get alarm by ID."""
        for alarm in self.data['alarms']:
            if alarm['id'] == alarm_id:
                return alarm
        return None
    
    def get_alarms_by_token(self, token: str) -> List[Dict[str, Any]]:
        """Get all active alarms for a specific token."""
        token = token.upper()
        return [alarm for alarm in self.data['alarms'] if alarm['token'] == token and alarm['status'] == 'active']
    
    def get_all_active_alarms(self) -> List[Dict[str, Any]]:
        """Get all active alarms."""
        return [alarm for alarm in self.data['alarms'] if alarm['status'] == 'active']
    
    def get_recent_triggered_alarms(self, limit: int = 10) -> List[Dict[str, Any]]:
        """Get recently triggered alarms."""
        return self.data['triggered_alarms'][-limit:] if self.data['triggered_alarms'] else []
    
    def remove_alarm(self, alarm_id: int) -> bool:
        """Remove an alarm by ID."""
        for i, alarm in enumerate(self.data['alarms']):
            if alarm['id'] == alarm_id:
                removed_alarm = self.data['alarms'].pop(i)
                self.data['last_update'] = datetime.now().isoformat()
                self._save_alarms()
                logger.info(f"Removed alarm {alarm_id}: {removed_alarm['token']} @ ${removed_alarm['target_price']}")
                return True
        return False
    
    def trigger_alarm(self, alarm_id: int, triggered_price: float) -> Optional[Dict[str, Any]]:
        """Mark an alarm as triggered."""
        for alarm in self.data['alarms']:
            if alarm['id'] == alarm_id and alarm['status'] == 'active':
                alarm['status'] = 'triggered'
                alarm['triggered_at'] = datetime.now().isoformat()
                alarm['triggered_price'] = triggered_price
                
                # Move to triggered alarms list
                self.data['triggered_alarms'].append(alarm.copy())
                
                # Keep only last 50 triggered alarms
                if len(self.data['triggered_alarms']) > 50:
                    self.data['triggered_alarms'] = self.data['triggered_alarms'][-50:]
                
                self.data['last_update'] = datetime.now().isoformat()
                self._save_alarms()
                
                logger.info(f"Triggered alarm {alarm_id}: {alarm['token']} @ ${triggered_price}")
                return alarm
        return None
    
    def check_alarms(self, price_data: Dict[str, float]) -> List[Dict[str, Any]]:
        """Check all active alarms against current prices."""
        triggered_alarms = []
        
        for alarm in self.get_all_active_alarms():
            token = alarm['token']
            target_price = alarm['target_price']
            direction = alarm['direction']
            
            # Get current price for this token
            current_price = price_data.get(token)
            if current_price is None:
                continue
            
            # Check if alarm should trigger
            should_trigger = False
            
            if direction == "above" and current_price >= target_price:
                should_trigger = True
            elif direction == "below" and current_price <= target_price:
                should_trigger = True
            
            if should_trigger:
                triggered_alarm = self.trigger_alarm(alarm['id'], current_price)
                if triggered_alarm:
                    triggered_alarms.append(triggered_alarm)
        
        return triggered_alarms
    
    def format_alarm_list(self, alarms: List[Dict[str, Any]], title: str = "Price Alarms") -> str:
        """Format a list of alarms for display."""
        if not alarms:
            return f"📢 <b>{title}</b>\n\n📭 No alarms found."
        
        message = f"📢 <b>{title}</b>\n\n"
        
        for alarm in alarms:
            status_emoji = "✅" if alarm['status'] == 'active' else "🔔"
            direction_emoji = "📈" if alarm['direction'] == 'above' else "📉"
            
            message += f"{status_emoji} <b>ID {alarm['id']}</b> - {alarm['token']}\n"
            message += f"   {direction_emoji} Alert {alarm['direction']} ${alarm['target_price']:,.2f}\n"
            
            if alarm['status'] == 'active':
                created_date = datetime.fromisoformat(alarm['created_at']).strftime('%m/%d %H:%M')
                message += f"   📅 Created: {created_date}\n"
            else:
                triggered_date = datetime.fromisoformat(alarm['triggered_at']).strftime('%m/%d %H:%M')
                message += f"   🔔 Triggered: {triggered_date} @ ${alarm['triggered_price']:,.2f}\n"
            
            message += "\n"
        
        if title == "Price Alarms":
            message += f"💡 <b>Commands:</b>\n"
            message += f"• <code>/alarm BTC 50000</code> - Set new alarm\n"
            message += f"• <code>/alarm BTC</code> - Show BTC alarms\n"
            message += f"• <code>/alarm 3</code> - Remove alarm ID 3"
        
        return message.strip()
    
    def format_triggered_alarm(self, alarm: Dict[str, Any]) -> str:
        """Format a triggered alarm notification."""
        direction_emoji = "📈" if alarm['direction'] == 'above' else "📉"
        price_change = alarm['triggered_price'] - alarm['current_price_when_set']
        change_percent = (price_change / alarm['current_price_when_set']) * 100
        
        message = f"""
🔔 <b>Price Alert Triggered!</b>

📊 <b>Alarm Details:</b>
• Token: {alarm['token']}
• Alert ID: {alarm['id']}
• Target Price: ${alarm['target_price']:,.2f}
• Direction: {alarm['direction'].upper()}

💰 <b>Price Info:</b>
• Current Price: ${alarm['triggered_price']:,.2f}
• Price When Set: ${alarm['current_price_when_set']:,.2f}
• Change: ${price_change:,.2f} ({change_percent:+.2f}%)

{direction_emoji} <b>Alert Condition:</b> Price moved {alarm['direction']} ${alarm['target_price']:,.2f}

⏰ <b>Triggered:</b> {datetime.now().strftime('%H:%M:%S')}

✅ <b>Status:</b> Alarm completed and removed from active list
        """
        
        return message.strip()
    
    def get_statistics(self) -> Dict[str, Any]:
        """Get alarm statistics."""
        active_alarms = self.get_all_active_alarms()
        triggered_alarms = self.data.get('triggered_alarms', [])
        
        # Count by token
        token_counts = {}
        for alarm in active_alarms:
            token = alarm['token']
            token_counts[token] = token_counts.get(token, 0) + 1
        
        return {
            'total_active': len(active_alarms),
            'total_triggered': len(triggered_alarms),
            'tokens_tracked': len(token_counts),
            'token_breakdown': token_counts,
            'next_id': self.data['next_id']
        } 